unit Account;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.StdCtrls,
  Vcl.ComCtrls,
  VAXSIPUSERAGENTCOMLib_TLB;

type
  TFormAccount = class(TForm)
    GroupAccount: TGroupBox;
    LabelUserName: TLabel;
    LabelDisplayName: TLabel;
    LabelDomain: TLabel;
    LabelPasswd: TLabel;
    LabelLogin: TLabel;
    LabelSIP: TLabel;
    LabelTotalLines: TLabel;
    EditAccountUserName: TEdit;
    EditAccountDisplayName: TEdit;
    EditAccountDomain: TEdit;
    EditAccountAuthPassword: TEdit;
    BtnOnline: TButton;
    EditAccountAuthLogin: TEdit;
    CheckRegisterToProxy: TCheckBox;
    EditAccountSIPProxy: TEdit;
    EditTotalLine: TEdit;
    CheckCaptureSoundDevice: TCheckBox;
    GroupAccountLog: TGroupBox;
    ListAccountLog: TListBox;
    BtnClearAccountLog: TButton;

    constructor Create(objTabSheetAccount: TTabSheet; objVaxCOM: TVaxVoIPSIP);overload;
    procedure BtnOnlineClick(Sender: TObject);
    procedure BtnClearAccountLogClick(Sender: TObject);

    procedure AddToStatusLog(LogText : String);

    procedure OnTryingToRegister();
    procedure OnSuccessToRegister();
    procedure OnFailToRegister();
    procedure OnFailToRegisterEx(nStatusCode: Integer; const sReasonPhrase: WideString);

    procedure OnTryingToReRegister();
    procedure OnSuccessToReRegister();
    procedure OnFailToReRegister();
    procedure OnFailToReRegisterEx(nStatusCode: Integer; const sReasonPhrase: WideString);

    procedure OnTryingToUnRegister();
    procedure OnFailToUnRegister();
    procedure OnSuccessToUnRegister();

    private
      m_objVaxCOM : TVaxVoIPSIP;

  end;


implementation

{$R *.dfm}

procedure TFormAccount.BtnClearAccountLogClick(Sender: TObject);
begin
    ListAccountLog.Clear();
end;

procedure TFormAccount.BtnOnlineClick(Sender: TObject);
var
  nTotalLine : Integer;
  bResult : WordBool;

begin
  if BtnOnline.Caption = 'Offline' Then
  begin
    BtnOnline.Caption := 'Online';
    m_objVaxCOM.UnInitialize();
    EditTotalLine.Enabled := True;
    Exit;
  end;

  if EditAccountUserName.Text = '' Then
  begin
    application.MessageBox('Please enter the SIP proxy Login Id.', 'VaxVoIP SDK', MB_ICONEXCLAMATION);
    Exit;
  end;

  if EditAccountSIPProxy.Text = '' Then
  begin
    application.MessageBox('Please enter the SIP proxy address or URI.', 'VaxVoIP SDK', MB_ICONEXCLAMATION);
    Exit;
  end;

  nTotalLine := StrToInt(EditTotalLine.Text);

  if nTotalLine <= 0 Then
  begin
    application.MessageBox('Total number of lines must be greater than 0.', 'VaxVoIP SDK', MB_ICONEXCLAMATION);
    Exit;
  end;

  m_objVaxCOM.SetLicenceKey('TRIAL-LICENSE-KEY');

  bResult := m_objVaxCOM.VaxInitObject(EditAccountUserName.Text, EditAccountAuthLogin.Text, EditAccountAuthPassword.Text, EditAccountDisplayName.Text, EditAccountDomain.Text, EditAccountSIPProxy.Text, CheckCaptureSoundDevice.Checked, nTotalLine);

  if bResult = False Then
    Exit;

  if CheckRegisterToProxy.Checked = True Then
  begin

  if False = m_objVaxCOM.RegisterToProxy(1800) Then
    Exit;

  AddToStatusLog('Registering to SIP Proxy.');

  end;

  m_objVaxCOM.EnableKeepAlive(10);

  BtnOnline.Caption := 'Offline';
  EditTotalLine.Enabled := False;

end;

procedure TFormAccount.AddToStatusLog(LogText : String);
begin
  ListAccountLog.Items.Add(LogText);
  ListAccountLog.TopIndex := ListAccountLog.Count - 1;
end;

constructor TFormAccount.Create(objTabSheetAccount: TTabSheet; objVaxCOM: TVaxVoIPSIP);
begin
  inherited Create(objTabSheetAccount);
  m_objVaxCOM := objVaxCOM;
end;

 /////////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////

procedure TFormAccount.OnTryingToRegister();
begin
  AddToStatusLog('Trying to Register.');
end;

procedure TFormAccount.OnSuccessToRegister();
begin
  AddToStatusLog('Success to Register.');
end;

procedure TFormAccount.OnFailToRegister();
begin
   AddToStatusLog('Fail to Register.');
end;

procedure TFormAccount.OnFailToRegisterEx(nStatusCode: Integer; const sReasonPhrase: WideString);
begin
  AddToStatusLog('Failed: ' + sReasonPhrase);
end;

procedure TFormAccount.OnTryingToReRegister();
begin
  AddToStatusLog('Trying to Re-Register.');
end;

procedure TFormAccount.OnSuccessToReRegister();
begin
  AddToStatusLog('Success to Re-Register.');
end;

procedure TFormAccount.OnFailToReRegister();
begin
  AddToStatusLog('Fail to Re-Register.');
end;

procedure TFormAccount.OnFailToReRegisterEx(nStatusCode: Integer; const sReasonPhrase: WideString);
begin
  AddToStatusLog('Failed' + sReasonPhrase);
end;

procedure TFormAccount.OnTryingToUnRegister();
begin
  AddToStatusLog('Trying to Unregister.');
end;

procedure TFormAccount.OnFailToUnRegister();
begin
  AddToStatusLog('Fail to Unregister.');
end;

procedure TFormAccount.OnSuccessToUnRegister();
begin
  AddToStatusLog('Unregistered successfully.');
end;

end.
