unit Chat;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.StdCtrls,
  Vcl.ComCtrls,
  VAXSIPUSERAGENTCOMLib_TLB,
  Vcl.ExtCtrls,
  StrUtils;

type
  TFormChat = class(TForm)
    GroupChatContact: TGroupBox;
    LabelChatStatus: TLabel;
    BtnAddChatContact: TButton;
    BtnRemoveChatContact: TButton;
    EditChatContact: TEdit;
    ComboMyChatStatus: TComboBox;
    GroupChatMsg: TGroupBox;
    LabelChatUserName: TLabel;
    LabelChatTyping: TLabel;
    ListChatMsg: TListBox;
    EditSendChatMsg: TEdit;
    EditSendChatUserName: TEdit;
    BtnSendChatMsg: TButton;
    GroupChatLog: TGroupBox;
    ListChatLog: TListBox;
    BtnClearChatLog: TButton;
    BtnClearChat: TButton;
    ChatTypingTimer: TTimer;
    ListViewChatContact: TListView;

    constructor Create(objTabSheetChat: TTabSheet; objVaxCOM: TVaxVoIPSIP);overload;
    procedure OnVaxInitialize();
    procedure OnVaxUnInitialize();
    procedure BtnAddChatContactClick(Sender: TObject);
    procedure BtnRemoveChatContactClick(Sender: TObject);
    procedure ComboMyChatStatusChange(Sender: TObject);
    procedure BtnSendChatMsgClick(Sender: TObject);
    procedure BtnClearChatLogClick(Sender: TObject);
    procedure BtnClearChatClick(Sender: TObject);
    procedure OnChatRecvMsgText( const sUserName, sMsgText: WideString);
    procedure OnChatRecvMsgTypingStart(const sUserName: WideString);
    procedure OnChatRecvMsgTypingStop();
    procedure OnChatSendMsgTextFail(const sUserName: WideString; nStatusCode: Integer; const sReasonPhrase,
                                    sMsgText: WideString; nUserValue32bit: Integer);
    procedure OnChatSendMsgTextSuccess(const sUserName, sMsgText: WideString; nUserValue32bit: Integer);
    procedure OnChatContactStatus(const sUserName: WideString; nStatusId: Integer);
    procedure ChatTypingTimerTimer(Sender: TObject);
    procedure EditSendChatMsgKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

  private
     m_objVaxCOM : TVaxVoIPSIP;
     m_bTyping : WordBool;
    procedure UpdateStatusCombo();
    function GetStatusText(nStatusId : Integer) : String;
    procedure IsTyping();
    procedure AddToChatLog(sChatLog: WideString);

  end;

implementation
 {$R *.dfm}

constructor TFormChat.Create(objTabSheetChat: TTabSheet; objVaxCOM: TVaxVoIPSIP);
var
Column : TListColumn;
begin
  inherited Create(objTabSheetChat);
  m_objVaxCOM := objVaxCOM;

  Column := ListViewChatContact.Columns.Add();
  Column.Caption := 'UserName';
  Column.Width := 90;

  Column := ListViewChatContact.Columns.Add();
  Column.Caption := 'Status';
  Column.Width := 110;


end;

procedure TFormChat.EditSendChatMsgKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var 
  sUserName: WideString;
begin
  sUserName := EditSendChatUserName.Text;
  m_objVaxCOM.ChatSendMessageTyping(sUserName, 1);

end;

procedure TFormChat.OnVaxInitialize();
begin
  UpdateStatusCombo();
end;

procedure TFormChat.OnVaxUnInitialize();
begin
  ListChatLog.Items.Clear();
  ListViewChatContact.Items.Clear();
end;

procedure TFormChat.UpdateStatusCombo();
begin

  ComboMyChatStatus.Clear();

  ComboMyChatStatus.Items.Add('Online');
  ComboMyChatStatus.Items.Add('Offline');
  ComboMyChatStatus.Items.Add('Away');
  ComboMyChatStatus.Items.Add('On the phone');
  ComboMyChatStatus.Items.Add('Busy');

  ComboMyChatStatus.ItemIndex := 0;

end;

procedure TFormChat.BtnAddChatContactClick(Sender: TObject);
var
Item : TListItem;

begin

  if EditChatContact.Text = '' then
    Exit;

  if m_objVaxCOM.ChatAddContact(EditChatContact.Text) = False then
    Exit;

   Item := ListViewChatContact.Items.Add;
   Item.Caption := EditChatContact.Text;
   Item.SubItems.Add('');


  if ListViewChatContact.ItemIndex = -1 then
    ListViewChatContact.ItemIndex := 0;

  EditChatContact.Text := '';

end;

procedure TFormChat.BtnClearChatClick(Sender: TObject);
begin
  ListChatMsg.Items.Clear()
end;

procedure TFormChat.BtnClearChatLogClick(Sender: TObject);
begin
  ListChatLog.Items.Clear();
end;

procedure TFormChat.BtnRemoveChatContactClick(Sender: TObject);
begin

  if ListViewChatContact.ItemIndex = -1 then
   Exit;

  if m_objVaxCOM.ChatRemoveContact(ListViewChatContact.Items[ListViewChatContact.ItemIndex].Caption) = False then
    Exit;

  ListViewChatContact.DeleteSelected;

end;

procedure TFormChat.BtnSendChatMsgClick(Sender: TObject);
begin
  if EditSendChatUserName.Text = '' then
  begin
    application.MessageBox('Please provide the user name', 'VaxVoIP SDK', MB_ICONEXCLAMATION);
    Exit;
  end;

  if m_objVaxCOM.ChatSendMessageText(EditSendChatUserName.Text, EditSendChatMsg.Text, 101, 0) = False then
    Exit;

  ListChatMsg.Items.Add('Me: ' + EditSendChatMsg.Text);
  EditSendChatMsg.Text := '';
  
end;

procedure TFormChat.ChatTypingTimerTimer(Sender: TObject);
begin
  IsTyping();
end;

procedure TFormChat.ComboMyChatStatusChange(Sender: TObject);
begin
  if ComboMyChatStatus.ItemIndex = -1 then
    Exit;

  m_objVaxCOM.ChatSetMyStatus(ComboMyChatStatus.ItemIndex);

end;

function TFormChat.GetStatusText(nStatusId : Integer) : String;
var
  sStatus : String;
begin
  sStatus := 'Offline';

  Case nStatusId of
    0: sStatus := 'Online';
    1: sStatus := 'Offline';
    2: sStatus := 'Away';
    3: sStatus := 'On the phone';
    4: sStatus := 'Busy';
  end;

  GetStatusText := sStatus;
end;

procedure TFormChat.IsTyping();
begin
  if m_bTyping = False Then
  begin
    OnChatRecvMsgTypingStop();
    ChatTypingTimer.Enabled := False;
  end
  Else
    m_bTyping := False

end;

procedure TFormChat.AddToChatLog(sChatLog: WideString);
begin
  ListChatLog.Items.Add(sChatLog);
  ListChatLog.TopIndex := ListChatLog.Items.Count - 1;
  
end;   

//////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

procedure TFormChat.OnChatContactStatus(const sUserName: WideString; nStatusId: Integer);
var
  sChatContact : String;
  nContactCount : Integer;
  Item : TListItem;
begin

  for nContactCount := 0 to (ListViewChatContact.Items.Count - 1) do
  begin
    sChatContact := UpperCase(ListViewChatContact.Items.Item[nContactCount].Caption);

    if UpperCase(sUserName) = sChatContact then
    begin
      Item := ListviewChatContact.items.Item[nContactCount];

      Item.SubItems.Strings[0] := ' [' + GetStatusText(nStatusId) + ']';
    end;

  end;

end;

procedure TFormChat.OnChatRecvMsgText(const sUserName, sMsgText: WideString);
var

  nStart : Integer;
  nEnd : Integer;

begin

  nStart := Pos('>', sMsgText);

  if nStart = 0 then
  begin
    ListChatMsg.Items.Add(sUserName + ': ' + sMsgText);
    exit;
  end;

  // Remote softphone is sending HTML based text messages

  nStart := nStart + 1;
  nEnd := PosEx('<', sMsgText, nStart);

  ListChatMsg.Items.Add(sUserName + ': ' + Copy(sMsgText, nStart, nEnd - nStart));
  ListChatMsg.ItemIndex := ListChatMsg.Items.Count - 1;

end;

procedure TFormChat.OnChatRecvMsgTypingStart(const sUserName: WideString);
begin
  m_bTyping := True;
  LabelChatTyping.Caption := sUserName + ' is typing message';

  ChatTypingTimer.Enabled := True;

  IsTyping();
end;

procedure TFormChat.OnChatRecvMsgTypingStop();
begin
  LabelChatTyping.Caption := '';
end;

procedure TFormChat.OnChatSendMsgTextFail(const sUserName: WideString; nStatusCode: Integer; const sReasonPhrase,
                                          sMsgText: WideString; nUserValue32bit: Integer);
begin
  AddToChatLog('Send Message Fail.');
end;

procedure TFormChat.OnChatSendMsgTextSuccess(const sUserName, sMsgText: WideString; nUserValue32bit: Integer);
begin
  AddToChatLog('Send Message Success.');
end;
end.
