unit Main;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.OleCtrls,
  Vcl.ComCtrls,
  Account,
  Phone,
  Chat,
  Setting,
  Vcl.OleServer,
  VAXSIPUSERAGENTCOMLib_TLB,
  Vcl.StdCtrls;

type

  TFormMain = class(TForm)
    PageControl1: TPageControl;
    m_objTabSheetAccount: TTabSheet;
    m_objTabSheetPhone: TTabSheet;
    m_objTabSheetChat: TTabSheet;
    m_objTabSheetSetting: TTabSheet;
    m_objVaxCOM: TVaxVoIPSIP;

    procedure FormCreate(Sender: TObject);
    procedure m_objVaxCOMTryingToRegister(Sender: TObject);
    procedure m_objVaxCOMSuccessToRegister(Sender: TObject);
    procedure m_objVaxCOMFailToRegister(Sender: TObject);
    procedure m_objVaxCOMFailToRegisterEx(ASender: TObject; StatusCode: Integer; const ReasonPhrase: WideString);
    procedure m_objVaxCOMTryingToReRegister(Sender: TObject);
    procedure m_objVaxCOMSuccessToReRegister(Sender: TObject);
    procedure m_objVaxCOMFailToReRegister(Sender: TObject);
    procedure m_objVaxCOMFailToReRegisterEx(ASender: TObject; StatusCode: Integer; const ReasonPhrase: WideString);
    procedure m_objVaxCOMTryingToUnRegister(Sender: TObject);
    procedure m_objVaxCOMFailToUnRegister(Sender: TObject);
    procedure m_objVaxCOMSuccessToUnRegister(Sender: TObject);
    procedure OnVaxInitialize();
    procedure OnVaxUnInitialize();
    procedure m_objVaxCOMFailToTransfer(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
    procedure m_objVaxCOMDTMFDigit(ASender: TObject; LineNo: Integer; const Digit: WideString);
    procedure m_objVaxCOMPlayWaveDone(ASender: TObject; LineNo: Integer);
    procedure m_objVaxCOMFailToConnect(ASender: TObject; LineNo: Integer);
    procedure m_objVaxCOMSuccessToConnect(ASender: TObject; LineNo: Integer; const ToRTPIP: WideString; ToRTPPort: Integer);
    procedure m_objVaxCOMConnecting(ASender: TObject; LineNo: Integer);
    procedure m_objVaxCOMIncomingCall(ASender: TObject; const CallId, DisplayName, UserName, FromURI, ToURI: WideString);
    procedure m_objVaxCOMIncomingCallRingingStart(ASender: TObject; const CallId: WideString);
    procedure m_objVaxCOMIncomingCallRingingStop(ASender: TObject; const CallId: WideString);
    procedure m_objVaxCOMCallTransferAccepted(ASender: TObject; LineNo: Integer);
    procedure m_objVaxCOMDisconnectCall(ASender: TObject; LineNo: Integer);
    procedure m_objVaxCOMProvisionalResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
    procedure m_objVaxCOMRedirectionResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase, Contact: WideString);
    procedure m_objVaxCOMRequestFailureResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
    procedure m_objVaxCOMServerFailureResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
    procedure m_objVaxCOMGeneralFailureResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
    procedure m_objVaxCOMDetectedAMD(ASender: TObject; LineNo : Integer; bIsHuman: Integer);
    Function GetActiveTab: Integer;
    procedure SetActiveTab(nTabIndex : Integer);
    procedure m_objVaxCOMChatContactStatus(ASender: TObject;
      const UserName: WideString; StatusId: Integer);
    procedure m_objVaxCOMChatRecvMsgText(ASender: TObject; const UserName, MsgText: WideString);
    procedure m_objVaxCOMChatRecvMsgTypingStart(ASender: TObject; const UserName: WideString);
    procedure m_objVaxCOMChatRecvMsgTypingStop(ASender: TObject; const UserName: WideString);
    procedure m_objVaxCOMChatSendMsgTextFail(ASender: TObject; const UserName: WideString; StatusCode: Integer;
                                            const ReasonPhrase, MsgText: WideString; UserValue32bit: Integer);
    procedure m_objVaxCOMChatSendMsgTextSuccess(ASender: TObject; const UserName, MsgText: WideString; UserValue32bit: Integer);

    private
      m_objFormAccount : TFormAccount;
      m_objFormPhone : TFormPhone;
      m_objFormChat : TFormChat;
      m_objFormSetting : TFormSetting;
  end;

var
  m_objFormMain : TFormMain;

implementation

{$R *.dfm}

procedure TFormMain.FormCreate(Sender: TObject);
begin
  m_objVaxCOM.SetMainForm(m_objFormMain);
  m_objFormAccount := TFormAccount.Create(m_objTabSheetAccount, m_objVaxCOM);
  m_objFormAccount.Parent := m_objTabSheetAccount;
  m_objFormAccount.Align := alClient;
  m_objFormAccount.Visible := true;

  m_objFormPhone := TFormPhone.Create(m_objTabSheetPhone, m_objVaxCOM, m_objFormMain);
  m_objFormPhone.Parent := m_objTabSheetPhone;
  m_objFormPhone.Align := alClient;
  m_objFormPhone.Visible := true;


  m_objFormChat := TFormChat.Create(m_objTabSheetChat, m_objVaxCOM);
  m_objFormChat.Parent := m_objTabSheetChat;
  m_objFormChat.Align := alClient;
  m_objFormChat.Visible := true;

  m_objFormSetting := TFormSetting.Create(m_objTabSheetSetting, m_objVaxCOM);
  m_objFormSetting.Parent := m_objTabSheetSetting;
  m_objFormSetting.Align := alClient;
  m_objFormSetting.Visible := true;

  PageControl1.ActivePage := m_objTabSheetAccount;

end;

  /////////////////////////////////////////////////////////////////////

procedure TFormMain.OnVaxInitialize();
begin
  m_objFormPhone.OnVaxInitialize();
  m_objFormChat.OnVaxInitialize();
  m_objFormSetting.OnVaxInitialize();
end;

procedure TFormMain.OnVaxUnInitialize();
begin
  m_objFormPhone.OnVaxUnInitialize();
  m_objFormChat.OnVaxUnInitialize();
  m_objFormSetting.OnVaxUnInitialize();
end;

procedure TFormMain.m_objVaxCOMFailToRegister(Sender: TObject);
begin
  m_objFormAccount.OnFailToRegister();
end;

procedure TFormMain.m_objVaxCOMFailToRegisterEx(ASender: TObject;
  StatusCode: Integer; const ReasonPhrase: WideString);
begin
  m_objFormAccount.OnFailToRegisterEx(StatusCode, ReasonPhrase);
end;

procedure TFormMain.m_objVaxCOMSuccessToRegister(Sender: TObject);
begin
  m_objFormAccount.OnSuccessToRegister();
end;

procedure TFormMain.m_objVaxCOMTryingToRegister(Sender: TObject);
begin
  m_objFormAccount.OnTryingToRegister();
end;

procedure TFormMain.m_objVaxCOMTryingToReRegister(Sender: TObject);
begin
  m_objFormAccount.OnTryingToReRegister();
end;

procedure TFormMain.m_objVaxCOMSuccessToReRegister(Sender: TObject);
begin
  m_objFormAccount.OnSuccessToReRegister();
end;

procedure TFormMain.m_objVaxCOMFailToReRegister(Sender: TObject);
begin
  m_objFormAccount.OnFailToReRegister();
end;

procedure TFormMain.m_objVaxCOMFailToReRegisterEx(ASender: TObject; StatusCode: Integer; const ReasonPhrase: WideString);
begin
  m_objFormAccount.OnFailToReRegisterEx(StatusCode, ReasonPhrase);
end;

procedure TFormMain.m_objVaxCOMTryingToUnRegister(Sender: TObject);
begin
  m_objFormAccount.OnTryingToUnRegister();
end;

procedure TFormMain.m_objVaxCOMFailToUnRegister(Sender: TObject);
begin
  m_objFormAccount.OnSuccessToReRegister();
end;

procedure TFormMain.m_objVaxCOMSuccessToUnRegister(Sender: TObject);
begin
  m_objFormAccount.OnSuccessToUnRegister();
end;

Function TFormMain.GetActiveTab: Integer;
begin
 Result := PageControl1.ActivePageIndex;
end;

procedure TFormMain.SetActiveTab(nTabIndex : Integer);
begin
  PageControl1.ActivePageIndex := nTabIndex;
end;
///////////////////////////////////////////////////////////////////////

procedure TFormMain.m_objVaxCOMFailToTransfer(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
begin
   m_objFormPhone.OnFailToTransfer(LineNo, StatusCode, ReasonPhrase);
end;

procedure TFormMain.m_objVaxCOMDTMFDigit(ASender: TObject; LineNo: Integer; const Digit: WideString);
begin
  m_objFormPhone.OnDTMFDigit(LineNo, Digit);
end;

procedure TFormMain.m_objVaxCOMPlayWaveDone(ASender: TObject; LineNo: Integer);
begin
  m_objFormPhone.OnPlayWaveDone(LineNo);
end;

procedure TFormMain.m_objVaxCOMFailToConnect(ASender: TObject; LineNo: Integer);
begin
  m_objFormPhone.OnFailToConnect(LineNo);
end;

procedure TFormMain.m_objVaxCOMSuccessToConnect(ASender: TObject; LineNo: Integer; const ToRTPIP: WideString; ToRTPPort: Integer);
begin
  m_objFormPhone.OnSuccessToConnect(LineNo, ToRTPIP, ToRTPPort);
  m_objFormSetting.OnSuccessToConnect(LineNo, ToRTPPort, ToRTPIP);
end;

procedure TFormMain.m_objVaxCOMConnecting(ASender: TObject; LineNo: Integer);
begin
  m_objFormPhone.OnConnecting(LineNo);
end;

procedure TFormMain.m_objVaxCOMIncomingCall(ASender: TObject; const CallId, DisplayName, UserName, FromURI, ToURI: WideString);
begin
  m_objFormPhone.OnIncomingCall(CallId, DisplayName, UserName, FromURI, ToURI);
end;

procedure TFormMain.m_objVaxCOMIncomingCallRingingStart(ASender: TObject; const CallId: WideString);
begin
  m_objFormPhone.OnIncomingRingingStart(CallId);
end;

procedure TFormMain.m_objVaxCOMIncomingCallRingingStop(ASender: TObject; const CallId: WideString);
begin
m_objFormPhone.OnIncomingCallRingingStop(CallId);
end;

procedure TFormMain.m_objVaxCOMCallTransferAccepted(ASender: TObject; LineNo: Integer);
begin
  m_objFormPhone.OnCallTransferAccepted(LineNo);
end;

procedure TFormMain.m_objVaxCOMDisconnectCall(ASender: TObject; LineNo: Integer);
begin
  m_objFormPhone.OnDisconnectCall(LineNo);
end;

procedure TFormMain.m_objVaxCOMProvisionalResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
begin
  m_objFormPhone.OnProvisionalResponse(LineNo, StatusCode, ReasonPhrase);
end;

procedure TFormMain.m_objVaxCOMRedirectionResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase,
                                                  Contact: WideString);
begin
  m_objFormPhone.OnRedirectionResponse(LineNo, StatusCode, ReasonPhrase,Contact);
end;

procedure TFormMain.m_objVaxCOMRequestFailureResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
begin
  m_objFormPhone.OnRequestFailureResponse(LineNo, StatusCode, ReasonPhrase);
end;

procedure TFormMain.m_objVaxCOMServerFailureResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
begin
  m_objFormPhone.OnServerFailureResponse(LineNo, StatusCode, ReasonPhrase);
end;

procedure TFormMain.m_objVaxCOMGeneralFailureResponse(ASender: TObject; LineNo, StatusCode: Integer; const ReasonPhrase: WideString);
begin
  m_objFormPhone.OnGeneralFailureResponse(LineNo, StatusCode, ReasonPhrase);
end;

procedure TFormMain.m_objVaxCOMDetectedAMD(ASender: TObject; LineNo : Integer; bIsHuman: Integer);
begin
  m_objFormPhone.OnDetectAMD(LineNo, bIsHuman);
end;

//////////////////////////////////////////////////////////////////

procedure TFormMain.m_objVaxCOMChatContactStatus(ASender: TObject; const UserName: WideString; StatusId: Integer);
begin
  m_objFormChat.OnChatContactStatus(UserName, StatusId);
end;

procedure TFormMain.m_objVaxCOMChatRecvMsgText(ASender: TObject; const UserName, MsgText: WideString);
begin
  m_objFormChat.OnChatRecvMsgText(UserName, MsgText);
end;

procedure TFormMain.m_objVaxCOMChatRecvMsgTypingStart(ASender: TObject; const UserName: WideString);
begin
  m_objFormChat.OnChatRecvMsgTypingStart(UserName);
end;

procedure TFormMain.m_objVaxCOMChatRecvMsgTypingStop(ASender: TObject; const UserName: WideString);
begin
  m_objFormChat.OnChatRecvMsgTypingStop();
end;

procedure TFormMain.m_objVaxCOMChatSendMsgTextFail(ASender: TObject; const UserName: WideString; StatusCode: Integer;
                                                  const ReasonPhrase, MsgText: WideString; UserValue32bit: Integer);
begin
  m_objFormChat.OnChatSendMsgTextFail(UserName, StatusCode, ReasonPhrase, MsgText, UserValue32bit);
end;

procedure TFormMain.m_objVaxCOMChatSendMsgTextSuccess(ASender: TObject; const UserName, MsgText: WideString;
                                                      UserValue32bit: Integer);
begin
  m_objFormChat.OnChatSendMsgTextSuccess(UserName, MsgText, UserValue32bit);
end;

end.
