unit Setting;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.StdCtrls,
  Vcl.ComCtrls,
  VAXSIPUSERAGENTCOMLib_TLB,
  ListData;

type
  TFormSetting = class(TForm)
    GroupVoiceCodec: TGroupBox;
    CheckGSM610: TCheckBox;
    CheckiLBC: TCheckBox;
    CheckG711A: TCheckBox;
    CheckG711U: TCheckBox;
    CheckG729: TCheckBox;
    GroupVoiceChanger: TGroupBox;
    RadioVCNormal: TRadioButton;
    RadioVCGranpa: TRadioButton;
    RadioVCTeen: TRadioButton;
    RadioVCHouseHold: TRadioButton;
    RadioVCHelium: TRadioButton;
    RadioVCChipmunk: TRadioButton;
    GroupOtherSetting: TGroupBox;
    CheckEchoCancel: TCheckBox;
    CheckMicBoost: TCheckBox;
    CheckDisturb: TCheckBox;
    CheckAGC: TCheckBox;
    CheckForceInbandDTMF: TCheckBox;
    GroupAudioIn: TGroupBox;
    ListAudioIn: TListBox;
    GroupAudioOut: TGroupBox;
    ListAudioOut: TListBox;

    procedure OnVaxInitialize();
    procedure OnVaxUnInitialize();
    constructor Create(objTabSheetSetting: TTabSheet; objVaxCOM: TVaxVoIPSIP);overload;
    procedure CheckGSM610Click(Sender: TObject);
    procedure CheckiLBCClick(Sender: TObject);
    procedure CheckG729Click(Sender: TObject);
    procedure CheckG711UClick(Sender: TObject);
    procedure CheckG711AClick(Sender: TObject);
    procedure RadioVCNormalClick(Sender: TObject);
    procedure RadioVCGranpaClick(Sender: TObject);
    procedure RadioVCTeenClick(Sender: TObject);
    procedure RadioVCHouseHoldClick(Sender: TObject);
    procedure RadioVCHeliumClick(Sender: TObject);
    procedure RadioVCChipmunkClick(Sender: TObject);
    procedure CheckEchoCancelClick(Sender: TObject);
    procedure CheckAGCClick(Sender: TObject);
    procedure CheckMicBoostClick(Sender: TObject);
    procedure CheckDisturbClick(Sender: TObject);
    procedure CheckForceInbandDTMFClick(Sender: TObject);
    procedure OnSuccessToConnect(nLineNo, nToRTPIP : Integer; const sToRTPPort : WideString);
    procedure ListAudioInClick(Sender: TObject);
    procedure ListAudioOutClick(Sender: TObject);

    private
      m_objVaxCOM : TVaxVoIPSIP;
      procedure UpdateAudioIn();
      procedure UpdateAudioOut();

  end;

implementation

{$R *.dfm}
procedure TFormSetting.OnVaxInitialize();
begin
  UpdateAudioIn();
  UpdateAudioOut();

  CheckEchoCancel.Checked := True;

  CheckG711U.Checked := True;
  CheckG729.Checked := True;
  CheckG711A.Checked := True;
  CheckGSM610.Checked := True;
  CheckiLBC.Checked := True;

  RadioVCNormal.Checked := True;

end;

Procedure TFormSetting.OnVaxUnInitialize;
begin

  ListAudioIn.Items.Clear();
  ListAudioOut.Items.Clear();
end;

procedure TFormSetting.RadioVCChipmunkClick(Sender: TObject);
begin
  m_objVaxCOM.VoiceChanger(20);
end;

procedure TFormSetting.RadioVCGranpaClick(Sender: TObject);
begin
  m_objVaxCOM.VoiceChanger(4);
end;

procedure TFormSetting.RadioVCHeliumClick(Sender: TObject);
begin
  m_objVaxCOM.VoiceChanger(16);
end;

procedure TFormSetting.RadioVCHouseHoldClick(Sender: TObject);
begin
  m_objVaxCOM.VoiceChanger(12);
end;

procedure TFormSetting.RadioVCNormalClick(Sender: TObject);
begin
  m_objVaxCOM.VoiceChanger(-1);
end;

procedure TFormSetting.RadioVCTeenClick(Sender: TObject);
begin
  m_objVaxCOM.VoiceChanger(8);
end;

procedure TFormSetting.UpdateAudioIn();
var
  nDeviceIndex : Integer;
  nDeviceId : Integer;
  sDeviceName : WideString;
  objTListData : TListData;
begin
  ListAudioIn.Clear();

  ListAudioIn.AddItem('Default Device', TListData.Create(-1, 'Default Device'));

  for nDeviceIndex := 0 to m_objVaxCOM.GetAudioInDevTotal() - 1 do
  begin
    nDeviceId := m_objVaxCOM.GetAudioInDevId(nDeviceIndex);
    sDeviceName := m_objVaxCOM.GetAudioInDevName(nDeviceIndex);
    ListAudioIn.AddItem(sDeviceName,TListData.Create(nDeviceId, sDeviceName));
  end;

  ListAudioIn.ItemIndex := 0;
  objTListData := TListData(ListAudioIn.Items.Objects[0]);
  nDeviceId := objTListData.m_nData;
  m_objVaxCOM.SetActiveAudioInDevice(nDeviceId);

end;

procedure TFormSetting.UpdateAudioOut();
var
  nDeviceIndex : Integer;
  nDeviceId : Integer;
  sDeviceName : WideString;
  objListData : TListData;
begin
  ListAudioOut.Clear();
  ListAudioOut.AddItem('Default Device', TListData.Create(-1, 'Default Device'));

  for nDeviceIndex := 0 to m_objVaxCOM.GetAudioOutDevTotal() - 1 do
  begin
    nDeviceId := m_objVaxCOM.GetAudioOutDevId(nDeviceIndex);
    sDeviceName := m_objVaxCOM.GetAudioOutDevName(nDeviceIndex);
    ListAudioOut.AddItem(sDeviceName,TListData.Create(nDeviceId, sDeviceName));
  end;

  ListAudioOut.ItemIndex := 0;
  objListData := TListData(ListAudioOut.Items.Objects[0]);
  nDeviceId := objListData.m_nData;
  m_objVaxCOM.SetActiveAudioOutDevice(nDeviceId);

end;

procedure TFormSetting.CheckAGCClick(Sender: TObject);
begin
  if CheckAGC.Checked = True Then
    m_objVaxCOM.EnableAGC(8000)
  else
    m_objVaxCOM.DisableAGC;
end;

procedure TFormSetting.CheckDisturbClick(Sender: TObject);
begin
  if CheckDisturb.Checked = True Then
    m_objVaxCOM.EnableDonotDisturb()
  else
    m_objVaxCOM.DisableDonotDisturb();
end;

procedure TFormSetting.CheckEchoCancelClick(Sender: TObject);
begin

  if CheckEchoCancel.Checked = True Then
    m_objVaxCOM.EnableEchoNoiseCancellation()
  else
    m_objVaxCOM.DisableEchoNoiseCancellation();

end;

procedure TFormSetting.CheckForceInbandDTMFClick(Sender: TObject);
var
  nLineCount : Integer;
  nTotalLine : Integer;
begin

  nTotalLine := m_objVaxCOM.GetTotalLine;

  For nLineCount := 0 To (nTotalLine - 1) do
  begin
    if CheckForceInbandDTMF.Checked Then
      m_objVaxCOM.EnableForceInbandDTMF(nLineCount)
    else
      m_objVaxCOM.DisableForceInbandDTMF(nLineCount);
  end;

end;

procedure TFormSetting.CheckGSM610Click(Sender: TObject);
begin
 {****Codec No 0 represents GSM 6.10 *****}

  if CheckGSM610.Checked = True Then
    m_objVaxCOM.SelectVoiceCodec(0)
  else
    m_objVaxCOM.DeselectVoiceCodec(0);
end;

procedure TFormSetting.CheckiLBCClick(Sender: TObject);
begin
{****Codec No 1 represents iLBC *****}

  if CheckiLBC.Checked = True Then
    m_objVaxCOM.SelectVoiceCodec(1)
  else
    m_objVaxCOM.DeselectVoiceCodec(1);
end;

procedure TFormSetting.CheckG711AClick(Sender: TObject);
begin
{****Codec No 2 represents iLBC *****}

  if CheckG711A.Checked = True Then
    m_objVaxCOM.SelectVoiceCodec(2)
  else
    m_objVaxCOM.DeselectVoiceCodec(2);
end;

procedure TFormSetting.CheckG711UClick(Sender: TObject);
begin
{****Codec No 3 represents iLBC *****}

  if CheckG711U.Checked = True Then
    m_objVaxCOM.SelectVoiceCodec(3)
  else
    m_objVaxCOM.DeselectVoiceCodec(3);
end;

procedure TFormSetting.CheckG729Click(Sender: TObject);
begin
{****Codec No 4 represents G729 *****}
  if CheckG729.Checked = True Then
    m_objVaxCOM.SelectVoiceCodec(4)
  else
    m_objVaxCOM.DeselectVoiceCodec(4);
end;

procedure TFormSetting.CheckMicBoostClick(Sender: TObject);
begin
  if CheckMicBoost.Checked = True Then
    m_objVaxCOM.EnableMicBoost()
  else
    m_objVaxCOM.DisableMicBoost();
end;

constructor TFormSetting.Create(objTabSheetSetting: TTabSheet; objVaxCOM: TVaxVoIPSIP);
begin
  inherited Create(objTabSheetSetting);
  m_objVaxCOM := objVaxCOM;
end;

procedure TFormSetting.ListAudioInClick(Sender: TObject);
var
  nItemIndex : Integer;
  nDeviceId : Integer;
  objListData : TListData;
begin
  nItemIndex := ListAudioIn.ItemIndex;
  objListData := TListData(ListAudioIn.Items.Objects[nItemIndex]);
  nDeviceId := objListData.m_nData;
  m_objVaxCOM.SetActiveAudioInDevice(nDeviceId);
end;

procedure TFormSetting.ListAudioOutClick(Sender: TObject);
var
  nItemIndex : Integer;
  nDeviceId : Integer;
  objListData : TListData;
begin
  nItemIndex := ListAudioOut.ItemIndex;
  objListData := TListData(ListAudioOut.Items.Objects[nItemIndex]);
  nDeviceId := objListData.m_nData;
  m_objVaxCOM.SetActiveAudioOutDevice(nDeviceId);
end;

procedure TFormSetting.OnSuccessToConnect(nLineNo, nToRTPIP : Integer; const sToRTPPort : WideString);
begin
  CheckMicBoost.Checked := m_objVaxCOM.IsMicBoostEnable();
end;
end.
