﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VaxVoIP
{
    public partial class ChatForm : Form
    {
        private cVaxVoIPCOM m_objVaxCOM; 
        private Boolean m_bTyping;

        public ChatForm(ref cVaxVoIPCOM objVaxCOM)
        {
            InitializeComponent();
            m_objVaxCOM = objVaxCOM;
        }

        public void OnVaxInitialize()
        {
            UpdateStatusCombo();
        }
        public void OnVaxUnInitialize()
        {
            ListChatLog.Items.Clear();
            ListBoxChatContact.Items.Clear();
        }

        private void UpdateStatusCombo()
        {
            ComboMyChatStatus.Items.Clear();

            ComboMyChatStatus.Items.Add("Online");
            ComboMyChatStatus.Items.Add("Offline");
            ComboMyChatStatus.Items.Add("Away");
            ComboMyChatStatus.Items.Add("On the phone");
            ComboMyChatStatus.Items.Add("Busy");

            ComboMyChatStatus.SelectedIndex = 0;
        }

        private void BtnAddChatContact_Click(object sender, EventArgs e)
        {
            if (EditChatContact.Text == "") return;

            Boolean bResult = m_objVaxCOM.ChatAddContact(EditChatContact.Text);

            if (!bResult) return;

            ListBoxChatContact.AddItem(EditChatContact.Text, EditChatContact.Text);

            if (ListBoxChatContact.SelectedIndex == -1)
                ListBoxChatContact.SelectedIndex = 0;

            EditChatContact.Text = "";
        }

        private void BtnRemoveChatContact_Click(object sender, EventArgs e)
        {
            if (ListBoxChatContact.SelectedIndex == -1) return;

            int nIndex = ListBoxChatContact.SelectedIndex;

            object objItemData = new object();
            ListBoxChatContact.GetItemData(nIndex, ref objItemData);

            Boolean bResult = m_objVaxCOM.ChatRemoveContact(Convert.ToString(objItemData));
            if (!bResult) return;

            ListBoxChatContact.RemoveItem(nIndex);
        }

        private void ComboMyChatStatus_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ComboMyChatStatus.SelectedIndex == -1) return;

            Boolean bResult = m_objVaxCOM.ChatSetMyStatus(ComboMyChatStatus.SelectedIndex);

            if(!bResult) return;            
        }

        private void BtnSendChatMsg_Click(object sender, EventArgs e)
        {
            if (EditSendChatUserName.Text.Length == 0)
            {
                MessageBox.Show("Please provide the user name");
                return;
            }

            Boolean bResult = m_objVaxCOM.ChatSendMessageText(EditSendChatUserName.Text, EditSendChatMsg.Text, 101, 0);
            if(!bResult) return;

            ListChatMsg.Items.Add("Me: " + EditSendChatMsg.Text);
            EditSendChatMsg.Text = "";
        }

        private void BtnClearChat_Click(object sender, EventArgs e)
        {
            ListChatMsg.Items.Clear();
        }

        private void AddToChatLog(String sChatLog)
        {
            ListChatLog.Items.Add(sChatLog);

            ListChatLog.TopIndex = ListChatLog.Items.Count - 1;
        }

        private string GetStatusText(int nStatusId)
        {
            string sStatus = "Offline";

            switch (nStatusId)
            {
                case 0:
                    sStatus = "Online";
                    break;

                case 1:
                    sStatus = "Offline";
                    break;

                case 2:
                    sStatus = "Away";
                    break;

                case 3:
                    sStatus = "On the phone";
                    break;

                case 4:
                    sStatus = "Busy";
                    break;
            }

            return sStatus;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////////

        public void OnChatContactStatus(String sUserName, int nStatusId)
        {
            for (int nContactCount = 0; nContactCount < ListBoxChatContact.Items.Count; nContactCount++)
            {
                String sChatContact = Convert.ToString(ListBoxChatContact.Items[nContactCount]);
                sChatContact = sChatContact.ToUpper();

                if (sChatContact.IndexOf(sUserName.ToUpper()) != -1)
                {
                    string sDisplayData = sUserName + " [" + GetStatusText(nStatusId) + "]";
                    ListBoxChatContact.UpdateItem(nContactCount, sDisplayData);
                }
            }
        }

        public void OnChatSendMsgTextSuccess(String sUserName, String sMsgText, int nUserValue32bit)
        {
            AddToChatLog(("Send Message Success."));
        }

        public void OnChatSendMsgTextFail(String sUserName, int nStatusCode, String sReasonPhrase, String sMsgText, int nUserValue32bit)
        {

            AddToChatLog(("Send Message Fail."));

        }

        public void OnChatSendMsgTypingSuccess(String sUserName, int nUserValue32bit)
        {
        }
        
        public void OnChatSendMsgTypingFail(String sUserName, int nStatusCode, String sReasonPhrase, int nUserValue32bit)
        {
        }

        public void OnChatRecvMsgText(String sUserName, String sMsgText)
        {
            int nstrStart = sMsgText.IndexOf(">");

            if(nstrStart == -1)
            {
                ListChatMsg.Items.Add((sUserName + ": " + sMsgText));
                return;
            }
        
            // Remote softphone is sending HTML based text messages

            nstrStart = nstrStart + 1;
            int nstrEnd = sMsgText.IndexOf("<", nstrStart);

            ListChatMsg.Items.Add((sUserName + ": " + sMsgText.Substring(nstrStart, nstrEnd - nstrStart)));
            ListChatMsg.TopIndex = ListChatMsg.Items.Count - 1;
        }

        public void OnChatRecvMsgTypingStart(String sUserName)
        {
            m_bTyping = true;
            LabelChatTyping.Text = sUserName + " is typing message";

            ChatTypingTimer.Start();

            IsTyping();
        }

        private void EditSendChatMsg_KeyDown(object sender, KeyEventArgs e)
        {
            String sUserName = EditSendChatUserName.Text;
            m_objVaxCOM.ChatSendMessageTyping(sUserName, 1);
        }

        private void ChatTypingTimer_Tick(object sender, EventArgs e)
        {
            IsTyping();
        }
           
        public void OnChatRecvMsgTypingStop()
        {
            LabelChatTyping.Text = "";
        }

        private void IsTyping()
        {
            if(!m_bTyping)
            {
                OnChatRecvMsgTypingStop();
                ChatTypingTimer.Stop();
            }
            else
                m_bTyping = false;
        }      

        private void BtnClearChatLog_Click(object sender, EventArgs e)
        {
             ListChatLog.Items.Clear();
        
        }

        private void ChatForm_Load(object sender, EventArgs e)
        {
            LabelChatTyping.Text = "";
        }       
    }
}
