﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace VaxVoIP
{
    public partial class PhoneForm : Form
    {
        private cVaxVoIPCOM m_objVaxCOM;
        private AlertForm m_objAlertForm;
        
        private Boolean m_bPlayingDialTone = false;
        private Boolean m_bPlayingRingTone = false;

        private const int SIP_RESPONSE_RINGING = 180;
        private const int SIP_RESPONSE_SESSION_PROGRESS = 183;
        private const int PHONE_TAB_PAGE_INDEX = 1;

        private int SND_ASYNC = 0x0001;
        private int SND_NODEFAULT = 0x0002;
        private int SND_LOOP = 0x0008;
        private int SND_FILENAME = 0x00020000;

        private Boolean[] m_arrayMuteLineMIC;
        private Boolean[] m_arrayMuteLineSPK;

        private Boolean[] m_arrayAllowListenConf;
        private Boolean[] m_arrayAllowSpeakConf;

        public Boolean[] m_arrayListenLeftSpk;
        private Boolean[] m_arrayListenRightSpk;

        private Boolean m_bPlayDialTone;
        

        [DllImport("winmm.dll")]
        private static extern bool PlaySound(string lpszName, int hModule, int dwFlags); 

        public PhoneForm(ref cVaxVoIPCOM objVaxCOM)
        {
            InitializeComponent();
            m_objVaxCOM = objVaxCOM;
        }

        public void OnVaxInitialize()
        {
            TimerTick.Enabled = true;
            ProgressTimer.Enabled = true;

            int nTotalLine = m_objVaxCOM.GetTotalLine();

            m_bPlayDialTone = CheckPlayDialTone.Checked;

            m_arrayMuteLineSPK = new Boolean[nTotalLine];
            m_arrayMuteLineMIC = new Boolean[nTotalLine];

            m_arrayListenLeftSpk = new Boolean[nTotalLine];
            m_arrayListenRightSpk = new Boolean[nTotalLine];

            m_arrayAllowListenConf = new Boolean[nTotalLine];
            m_arrayAllowSpeakConf = new Boolean[nTotalLine];

            for (int nLineNo = 0; nLineNo < nTotalLine; nLineNo++)
            {
                m_arrayMuteLineSPK[nLineNo] = false;
                m_arrayMuteLineMIC[nLineNo] = false;

                m_arrayListenLeftSpk[nLineNo] = true;
                m_arrayListenRightSpk[nLineNo] = true;

                m_arrayAllowListenConf[nLineNo] = true;
                m_arrayAllowSpeakConf[nLineNo] = true;

                ComboConsultLineA.Items.Add("Line-" + Convert.ToString(nLineNo + 1));
                ComboConsultLineB.Items.Add("Line-" + Convert.ToString(nLineNo + 1));
            }

            FillListLines();

            CheckLeftSpkListen.Checked = true;
            CheckRightSpkListen.Checked = true;

            CheckAllowListenConf.Checked = true;
            CheckAllowSpeakConf.Checked = true;

            CheckMuteLineSPK.Checked = false;
            CheckMuteLineMIC.Checked = false;

            EditForwardCallPhoneNo.Text = " ";
            CheckForwardCall.Checked = false;
            EditForwardCallPhoneNo.Text = "";
        }

        public void OnVaxUnInitialize()
        {
            TimerTick.Enabled = false;
            ProgressTimer.Enabled = false;
            
            SpkProgBar.Value = 0;
            MicProgBar.Value = 0;
            
            ListLines.Items.Clear();
            ListStatusLog.Items.Clear();
            ListIncomingCall.Items.Clear();
        }

        private void FillListLines()
        {
            int nTotalLine = m_objVaxCOM.GetTotalLine();

            for (int nLineNo = 0; nLineNo < nTotalLine; nLineNo++)
            {
                ListLines.Items.Add("Line-" + Convert.ToString(nLineNo + 1) + " : FREE");
            }

            ListLines.SelectedIndex = 0;    
        }

        private void UpdateListLine()
        {
            int nSelectedLineNo = ListLines.SelectedIndex;
            int nTotalLine = m_objVaxCOM.GetTotalLine();

            if (CheckStartConf.Checked)
            {
                for (int nLineCount = 0; nLineCount < nTotalLine; nLineCount++)
                {
                    if (m_objVaxCOM.IsLineHold(nLineCount))
                        m_objVaxCOM.UnHoldLine(nLineCount);
                }
            }
            else
            {
                for (int nLineCount = 0; nLineCount < nTotalLine; nLineCount++)
                {
                    if (nLineCount == nSelectedLineNo) continue;

                    if (m_objVaxCOM.IsLineConnected(nLineCount) == true && m_objVaxCOM.IsLineHold(nLineCount) == false)                   
                    {
                        m_objVaxCOM.HoldLine(nLineCount);
                    }
                }

                if (m_objVaxCOM.IsLineHold(nSelectedLineNo))
                    m_objVaxCOM.UnHoldLine(nSelectedLineNo);
            }
        }	

        private void ShowListViewLineStatus(int nLineNo, String sStatus)
        {
            ListLines.Items[nLineNo] = ListLines.Items[nLineNo] = "Line-" + Convert.ToString(nLineNo + 1) + " : " +sStatus;
        }

        private void TimerTick_Tick(object sender, EventArgs e)
        {
            int nTotalLine = m_objVaxCOM.GetTotalLine();

            UpdateListLine();
           
            for (int nLineNo = 0; nLineNo < nTotalLine; nLineNo++)
            {
                String sItemText = ListLines.Items[nLineNo].ToString();

                if (!m_objVaxCOM.IsLineOpen(nLineNo))
                {
                    if (sItemText.IndexOf("CLOSE") == -1)
                    {
                        ShowListViewLineStatus(nLineNo,"CLOSE");
                    }
                }
                else if (m_objVaxCOM.IsLineBusy(nLineNo))
                {
                   
                    if (m_objVaxCOM.IsWaveFilePlaying(nLineNo))
                    {
                        if (sItemText.IndexOf("PLAYING") == -1)
                        {
                            ShowListViewLineStatus(nLineNo, "PLAYING");                            
                        }
                    }
                    else if (m_objVaxCOM.IsLineHold(nLineNo))
                    {
                        if (sItemText.IndexOf("HOLD") == -1)
                        {
                            ShowListViewLineStatus(nLineNo, "HOLD");
                        }
                    }
                    else
                    {
                        if (sItemText.IndexOf("BUSY") == -1 && sItemText.IndexOf("CONNECTING") == -1)
                        {
                            ShowListViewLineStatus(nLineNo, "BUSY");
                        }
                    }
                }
                else
                {
                    if (sItemText.IndexOf("FREE") == -1)
                    {
                        ShowListViewLineStatus(nLineNo, "FREE");
                    }
                }
            }
        }

        private String CodecName(int nCodecNo)
        {
            if (nCodecNo == 0)
                return "gsm6.10";

            if (nCodecNo == 1)
                return "iLBC";

            if (nCodecNo == 2)
                return "gsm711a";

            if (nCodecNo == 3)
                return "gsm711u";

            if (nCodecNo == 4)
                return "g729";

            return "";
        }       

        private void ProgressTimer_Tick(object sender, EventArgs e)
        {
            SpkProgBar.Value = m_objVaxCOM.GetSpkSoundLevel();
            MicProgBar.Value = m_objVaxCOM.GetMicSoundLevel();
        }

        private void BtnDialPad1_Click(object sender, EventArgs e)
        {                   
            EditPhoneNo.Text += "1";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "1");
        }

        private void BtnDialPad2_Click(object sender, EventArgs e)
        {
            EditPhoneNo.Text += "2";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "2");
        }

        private void BtnDialPad3_Click(object sender, EventArgs e)
        {
            EditPhoneNo.Text += "3";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "3");
        }

        private void BtnDialPad4_Click(object sender, EventArgs e)
        {
            EditPhoneNo.Text += "4";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "4");
        }

        private void BtnDialPad5_Click(object sender, EventArgs e)
        {
            EditPhoneNo.Text += "5";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "5");
        }

        private void BtnDialPad6_Click(object sender, EventArgs e)
        {
            EditPhoneNo.Text += "6";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "6");
        }

        private void BtnDialPad7_Click(object sender, EventArgs e)
        {
            EditPhoneNo.Text += "7";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "7");
        }

        private void BtnDialPad8_Click(object sender, EventArgs e)
        {
            EditPhoneNo.Text += "8";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "8");
        }

        private void BtnDialPad9_Click(object sender, EventArgs e)
        {
            EditPhoneNo.Text += "9";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "9");
        }

        private void BtnDialPadStar_Click(object sender, EventArgs e)
        {
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "*");
        }

        private void BtnDialPad0_Click(object sender, EventArgs e)
        {
            EditPhoneNo.Text += "0";
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "0");
        }

        private void BtnDialPadNo_Click(object sender, EventArgs e)
        {
            m_objVaxCOM.DigitDTMF(ListLines.SelectedIndex, "#");
        }
        
        private void StartRingTone()
        {
            if(CheckPlayRingTone.Checked == false) return;
            
            if(m_bPlayingRingTone) return;
           
            PlaySound("ring.wav", 0, SND_ASYNC | SND_FILENAME | SND_LOOP | SND_NODEFAULT);
            m_bPlayingRingTone = true;
        }
        
        private void StopRingTone()
        {
            if(!m_bPlayingRingTone) return;
            
            PlaySound(null, 0, 0);
            m_bPlayingRingTone = false;
        }

        private void StartDialTone()
        {
            if(CheckPlayDialTone.Checked == false) return;
            
            if(m_bPlayingDialTone) return;
            
            PlaySound("ring.wav", 0, SND_ASYNC | SND_FILENAME | SND_LOOP | SND_NODEFAULT);
            m_bPlayingDialTone = true;
        }
        
        private void StopDialTone()
        {
            if(!m_bPlayingDialTone) return;
            
            PlaySound(null, 0, 0);
            m_bPlayingDialTone = false;
        }
      
        private int FindLine()
        {
            int nTotalLine = m_objVaxCOM.GetTotalLine();
            int nLineNo = ListLines.SelectedIndex;

            if (nLineNo == -1)
                return -1;

            if (m_objVaxCOM.IsLineBusy(nLineNo) == false)
            {
                return nLineNo;
            }
        
            for(nLineNo = 0; nLineNo < nTotalLine; nLineNo++)
            {
                if (m_objVaxCOM.IsLineBusy(nLineNo) == false)
                    return nLineNo;
            }

            return -1;
        }

        private void BtnDial_Click(object sender, EventArgs e)
        {
            if (EditPhoneNo.Text == "")
            {
                MessageBox.Show("Please enter a phone number.", "VaxVoIP SDK");
                return;
            }
            
            int nLineNo = FindLine();

            if (nLineNo == -1)
            {
                MessageBox.Show("No line is free");
                return;
            }           

            m_objVaxCOM.DetectAMD(nLineNo, CheckDetectAMD.Checked, 6000, 300, 2);

            Boolean bResult = m_objVaxCOM.DialCall(nLineNo, EditPhoneNo.Text, m_objVaxCOM.GetActiveAudioInDevice(), m_objVaxCOM.GetActiveAudioOutDevice());
            if (!bResult) return;
                         
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Dialing.");
            ShowListViewLineStatus(nLineNo, "CONNECTING");
        }

        private void AddToStatusLog(String sStatusLog)
        {
            ListStatusLog.Items.Add(sStatusLog);
            ListStatusLog.TopIndex = ListStatusLog.Items.Count - 1;
        }

        private void BtnHangUp_Click(object sender, EventArgs e)
        {
            int nLine = ListLines.SelectedIndex;
            m_objVaxCOM.Disconnect(nLine);
            StopDialTone();                
        }

        private void CheckPlayDialTone_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckPlayDialTone.Checked == false)
                StopDialTone();         
        }  

        private void BtnResetRecordingBuffer_Click(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            m_objVaxCOM.ResetRecording(nLineNo);
        }

        
        private void BtnSaveToWaveFile_Click(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            Boolean bResult = m_objVaxCOM.SaveRecordingToWaveFile(nLineNo, EditSaveFileName.Text);

            if(!bResult) return;
            
            MessageBox.Show("Saved Successfully", "VaxVoIP SDK");
        }

        private void BtnSetPlayFile_Click(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            m_objVaxCOM.PlayWaveOpen(nLineNo, EditPlayFileName.Text);                
        }

        private void BtnPlayRewind_Click(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            m_objVaxCOM.PlayWaveSkipTo(nLineNo, m_objVaxCOM.PlayWavePosition(nLineNo) - 2);                
        }

        private void BtnPlayForward_Click(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            m_objVaxCOM.PlayWaveSkipTo(nLineNo, m_objVaxCOM.PlayWavePosition(nLineNo) + 2);                
        }

        private void BtnStartPlay_Click(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            m_objVaxCOM.PlayWaveStart(nLineNo, CheckPlayListen.Checked);                
        }

        private void BtnPlayPause_Click(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            m_objVaxCOM.PlayWavePause(nLineNo);            
        }

        private void BtnPlayStop_Click(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            m_objVaxCOM.PlayWaveStop(nLineNo);            
        }

        private void BtnAccept_Click(object sender, EventArgs e)
        {
            int nSelectedItemIndex = ListIncomingCall.SelectedIndex;
            
            if (nSelectedItemIndex == -1) return;
            
           cListIncomingCall objcListIncomingCall = (cListIncomingCall) ListIncomingCall.Items[nSelectedItemIndex];

            String sCallId = objcListIncomingCall.m_sCallId;
            
            int nLineNo = FindLine();

            if (nLineNo == -1)
            {
                MessageBox.Show("No line is free");
                return;
            }

            m_objVaxCOM.DetectAMD(nLineNo, CheckDetectAMD.Checked, 6000, 300, 2);

            Boolean bResult = m_objVaxCOM.AcceptCall(nLineNo, sCallId, m_objVaxCOM.GetActiveAudioInDevice(), m_objVaxCOM.GetActiveAudioOutDevice());

            if (!bResult) return;
            
            ListIncomingCall.Items.Remove(objcListIncomingCall);

            ShowListViewLineStatus(nLineNo, "CONNECTING");

            if (ListIncomingCall.Items.Count > 0)
            {
                if(ListIncomingCall.SelectedIndex == -1)
                    ListIncomingCall.SelectedIndex = 0;

                return;
            }

            StopRingTone();
        }

        private void BtnReject_Click(object sender, EventArgs e)
        {
            int nSelectedItemIndex = ListIncomingCall.SelectedIndex;
            if (nSelectedItemIndex == -1) return;

            cListIncomingCall objcListIncomingCall = (cListIncomingCall)ListIncomingCall.Items[nSelectedItemIndex];

            String sCallId = objcListIncomingCall.m_sCallId;

            Boolean bResult = m_objVaxCOM.RejectCall(sCallId);
            
            if (!bResult) return;
            
            ListIncomingCall.Items.Remove(objcListIncomingCall);

            if (ListIncomingCall.Items.Count > 0)
            {
                if (ListIncomingCall.SelectedIndex == -1)
                    ListIncomingCall.SelectedIndex = 0;

                return;
            }

            StopRingTone();
        }

        private void BtnTransferCall_Click(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            m_objVaxCOM.TransferCallEx(nLineNo, EditTransferPhoneNo.Text);                
        }

        private void BtnConsultTransfer_Click(object sender, EventArgs e)
        {
            if (ComboConsultLineA.SelectedIndex == ComboConsultLineB.SelectedIndex)
            {
                MessageBox.Show("Please select two different lines.", "VaxVoIP SIP SDK");
                return;
            }

           m_objVaxCOM.JoinTwoLine(ComboConsultLineA.SelectedIndex, ComboConsultLineB.SelectedIndex);
        }

        private void BtnClearLog_Click(object sender, EventArgs e)
        {
            ListStatusLog.Items.Clear();
        }     

        private void CheckMuteLineSPK_CheckedChanged(object sender, EventArgs e)
        {            
            int nLineNo = ListLines.SelectedIndex;

            Boolean bResult = m_objVaxCOM.MuteLineSPK(nLineNo, CheckMuteLineSPK.Checked);

            if (!bResult) return;           

            m_arrayMuteLineSPK[nLineNo] = CheckMuteLineSPK.Checked;
        }

        private void CheckMuteLineMIC_CheckedChanged(object sender, EventArgs e)
        {            
            int nLineNo = ListLines.SelectedIndex;

            Boolean bResult = m_objVaxCOM.MuteLineMIC(nLineNo, CheckMuteLineMIC.Checked);

            if (!bResult) return;

            m_arrayMuteLineMIC[nLineNo] = CheckMuteLineMIC.Checked;
        }

        private int SpkListenChannel(Boolean bListenLeft, Boolean bListenRight)
        {
            if (bListenLeft == true && bListenRight == false)
                return 0;

            if (bListenLeft == false && bListenRight == true)
                return 1;

            return 2;
        }

        private void CheckLeftSpkListen_CheckedChanged(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            if (CheckLeftSpkListen.Checked == false && CheckRightSpkListen.Checked == false)
                CheckRightSpkListen.Checked = true;

            Boolean bResult = m_objVaxCOM.LineVoiceChannelSPK(nLineNo, SpkListenChannel(CheckLeftSpkListen.Checked, CheckRightSpkListen.Checked));

            if (!bResult) return;            

            m_arrayListenLeftSpk[nLineNo] = CheckLeftSpkListen.Checked;
        }

        private void CheckRightSpkListen_CheckedChanged(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            if (CheckLeftSpkListen.Checked == false && CheckRightSpkListen.Checked == false)
                CheckLeftSpkListen.Checked = true;

            Boolean bResult = m_objVaxCOM.LineVoiceChannelSPK(nLineNo, SpkListenChannel(CheckLeftSpkListen.Checked, CheckRightSpkListen.Checked));

            if (!bResult) return;                   

            m_arrayListenRightSpk[nLineNo] = CheckRightSpkListen.Checked;
        }      

        private void CheckSpkMute_CheckedChanged(object sender, EventArgs e)
        {
            m_objVaxCOM.MuteSpk(CheckSpkMute.Checked);        
        }

        private void CheckMuteMic_CheckedChanged(object sender, EventArgs e)
        {
            m_objVaxCOM.MuteMic(CheckMuteMic.Checked);
        }

        private void SliderSpkVol_Scroll(object sender, EventArgs e)
        {
            /*
            SetSpkVolume method requires Volume value from 0 to 255 range    
            but the slider range is 0 to 10, so we multiply the slider value 
            to 25 to get the required value to pass to SetSpkVolume method.
            */

            m_objVaxCOM.SetSpkVolume(SliderSpkVol.Value * 25);
        }

        private void SliderMicVol_Scroll(object sender, EventArgs e)
        {
            /*
           SetMicVolume method requires Volume value from 0 to 255 range    
           but the slider range is 0 to 10, so we multiply the slider value 
           to 25 to get the required value to pass to SetMicVolume method.
           */

            m_objVaxCOM.SetMicVolume(SliderMicVol.Value * 25);
        }

        private String GetCallerId(String sDisplayName, String sUserName)
        {
            if(sDisplayName.Length != 0)
                return sDisplayName;
            
            return sUserName;
        }

        private Boolean FindCallId(String sCallId)
        {
            cListIncomingCall objListIncomingCall;
            
            for(int nListIndex = 0; nListIndex < ListIncomingCall.Items.Count; nListIndex++)
            {
                objListIncomingCall = (cListIncomingCall) ListIncomingCall.Items[nListIndex];

                if(sCallId == objListIncomingCall.m_sCallId)
                    return true;
            }
            
            return false;
        }
        
        private void RemoveCallIdFromList(String sCallId)
        {
            cListIncomingCall objListIncomingCall;
            
            for (int nListIndex = 0; nListIndex < ListIncomingCall.Items.Count; nListIndex++)
            {
                objListIncomingCall = (cListIncomingCall)ListIncomingCall.Items[nListIndex];

                if (sCallId == objListIncomingCall.m_sCallId)
                {
                    ListIncomingCall.Items.Remove(objListIncomingCall);
                    objListIncomingCall = null;
                    return;
                }
            }
        }
        ////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////


        public void OnFailToTransfer(int nLineNo, int nStatusCode, String sReasonPhrase)
        {
            AddToStatusLog("Fail to Transfer: " + sReasonPhrase);            
        }

        public void OnDTMFDigit(int nLineNo, String sDigit)
        {
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Key-pressed: " + sDigit);
        }

        public void OnPlayWaveDone(int nLineNo)
        {
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Play wave done.");
        }

        public void OnFailToConnect(int nLineNo)
        {
            StopDialTone();
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Fail to connect.");
        }

        public void OnSuccessToConnect(int nLineNo, String sToRTPIP, int nToRTPPort)
        {
            StopDialTone();

            if (nLineNo != ListLines.SelectedIndex)
            {
                m_objVaxCOM.HoldLine(nLineNo);
                ListLines.Items[nLineNo] = "Line-" + Convert.ToString(nLineNo + 1) + "  :  HOLD";
            }
            else
            {
                ListLines.Items[nLineNo] = "Line-" + Convert.ToString(nLineNo + 1) + "  :  BUSY";
            }            

            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Success to connect.");

            int nVol = m_objVaxCOM.GetSpkVolume();
            SliderSpkVol.Value = nVol / 25;

            nVol = m_objVaxCOM.GetMicVolume();
            SliderMicVol.Value = nVol / 25;
        }

        public void OnConnecting(int nLineNo)
        {
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Connecting.");
        }

        public void OnIncomingCall(String sCallId, String sDisplayName, String sUserName, String sFromURI, String sToURI)
        {
            Boolean bResult = FindCallId(sCallId);

            if( bResult) return;
        
            String sCallerId = GetCallerId(sDisplayName, sUserName);

            ListIncomingCall.Items.Add(new cListIncomingCall(sCallId, sCallerId));

            if ((ListIncomingCall.SelectedIndex == -1) && (ListIncomingCall.Items.Count > 0))
                ListIncomingCall.SelectedIndex = 0;        
        }

        public void OnIncomingCallRingingStart(String sCallId)
        {   
            if(CheckPlayRingTone.Checked == true)
                StartRingTone();

            int nTabIndex = MainForm.m_objMainForm.GetActiveTab();

            if (nTabIndex != PHONE_TAB_PAGE_INDEX)
            {
                if(m_objAlertForm == null)
                {
                    m_objAlertForm = new AlertForm(this);
                    m_objAlertForm.ShowDialog();
                }
            }
        }

        public void OnIncomingCallRingingStop(String sCallId)
        {
            RemoveCallIdFromList(sCallId);
            
            if(ListIncomingCall.Items.Count > 0) return;
            
            StopRingTone();

            if(m_objAlertForm == null) return;

            m_objAlertForm.Close();
            m_objAlertForm = null;
        }
        
        public void OkAlert()
        {
            MainForm.m_objMainForm.SetActiveTab(PHONE_TAB_PAGE_INDEX);

            m_objAlertForm.Close();
            m_objAlertForm = null;
        }

        public void CancelAlert()
        {
            m_objAlertForm.Close();
            m_objAlertForm = null;
        }

        public void OnCallTransferAccepted(int nLineNo)
        {
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Transfer accepted.");
        }

        public void OnDisconnectCall(int nLineNo)
        {
            StopDialTone();
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Hung up.");
        }
        public void OnIncomingDiagnostic(String sMsgSIP, String sFromIP, int nFromPort)
        {
            
        }

        public void OnOutgoingDiagnostic(String sMsgSIP, String sToIP, int nToPort)
        {
            
        }

        public void OnVoiceMailMsg(int bIsMsgWaiting, int nNewMsgCount, int nOldMsgCount, int nNewUrgentMsgCount, int nOldUrgentMsgCount, String sMsgAccount)
        {
            
        }

        public void OnSessionLostEvent(int nLineNo)
        {
            
        }

        public void OnMsgNOTIFY(String sMsg)
        {
            
        }

        public void OnProvisionalResponse(int nLineNo, int nStatusCode, String sReasonPhrase)
        {
            if(nStatusCode == SIP_RESPONSE_RINGING)
                StartDialTone();
            
            else if(nStatusCode == SIP_RESPONSE_SESSION_PROGRESS)
                StopDialTone();

            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Response: " + sReasonPhrase);
        }

        public void OnRedirectionResponse(int nLineNo, int nStatusCode, String sReasonPhrase, String sContact)
        {
            StopDialTone();
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Response: " + sReasonPhrase);
        }

        public void OnRequestFailureResponse(int nLineNo, int nStatusCode, String sReasonPhrase)
        {
            StopDialTone();
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Response: " + sReasonPhrase);   
        }

        public void OnServerFailureResponse(int nLineNo, int nStatusCode, String sReasonPhrase)
        {
            StopDialTone();
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Response: " + sReasonPhrase);
        }

        public void OnGeneralFailureResponse(int nLineNo, int nStatusCode, String sReasonPhrase)
        {
            StopDialTone();
            AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Response: " + sReasonPhrase); 
        }

        public void OnSuccessToHold(int nLineNo)
        {
            
        }

        public void OnTryingToHold(int nLineNo)
        {
            
        }

        public void OnFailToHold(int nLineNo)
        {
            
        }

        public void OnSuccessToUnHold(int nLineNo)
        {
            
        }

        public void OnTryingToUnHold(int nLineNo)
        {
            
        }

        public void OnFailToUnHold(int nLineNo)
        {
            
        }

        public void OnDetectedAMD(int nLineNo, int bIsHuman)
        {
            if (Convert.ToBoolean(bIsHuman) == false)
                AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Answering Machine [Yes]");
            else
                AddToStatusLog("Line-" + Convert.ToString(nLineNo + 1) + ": Answering Machine [No]");        
        }

        private void CheckPlayRingTone_CheckedChanged(object sender, EventArgs e)
        {
            if (!CheckPlayRingTone.Checked)
                StopRingTone();
        }

        private void CheckForwardCall_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckForwardCall.Checked == true)
            {
                if (EditForwardCallPhoneNo.Text == "")
                {
                    MessageBox.Show("Please enter a phone number.");
                    CheckForwardCall.Checked = false;
                    return;
                }
            }

            m_objVaxCOM.ForwardCall(CheckForwardCall.Checked, EditForwardCallPhoneNo.Text);
        }

        private void ListLines_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;
            if (nLineNo == -1) return;

            CheckMuteLineMIC.Checked = m_arrayMuteLineMIC[nLineNo];
            CheckMuteLineSPK.Checked = m_arrayMuteLineSPK[nLineNo];

            CheckAllowListenConf.Checked = m_arrayAllowListenConf[nLineNo];
            CheckAllowSpeakConf.Checked = m_arrayAllowSpeakConf[nLineNo];

            CheckLeftSpkListen.Checked = m_arrayListenLeftSpk[nLineNo];
            CheckRightSpkListen.Checked = m_arrayListenRightSpk[nLineNo];

            CheckStartRecording.Checked = m_objVaxCOM.IsRecording(nLineNo);
        }

        private void CheckStartConf_CheckedChanged(object sender, EventArgs e)
        {
            int nBusyCount = 0;
            int nTotalLine = m_objVaxCOM.GetTotalLine();

            for (int nLineCount = 0; nLineCount < nTotalLine; nLineCount++)
            {
                if (m_objVaxCOM.IsLineBusy(nLineCount))
                    nBusyCount++;
            }

            if (nBusyCount < 2 && CheckStartConf.Checked)
            {
                CheckStartConf.Checked = false;
                MessageBox.Show("Dial/receive more than one calls and then click start conference check box.", "VaxVoIP SDK");
            }
        }             

        private void CheckStartRecording_CheckedChanged(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;
            if (CheckStartRecording.Checked == true)
            {
                if (m_objVaxCOM.IsLineBusy(nLineNo) == false)
                    MessageBox.Show("Selected Line is not in voice session.");

                m_objVaxCOM.StartRecording(nLineNo, 2, true);
            }
            else
                m_objVaxCOM.StopRecording(nLineNo);        
        }

        private void CheckAllowListenConf_CheckedChanged(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            Boolean bResult = m_objVaxCOM.ConfAllowLine(nLineNo, CheckAllowListenConf.Checked, CheckAllowSpeakConf.Checked);
            if (!bResult)
                return;

            m_arrayAllowListenConf[nLineNo] = CheckAllowListenConf.Checked;
        }

        private void CheckAllowSpeakConf_CheckedChanged(object sender, EventArgs e)
        {
            int nLineNo = ListLines.SelectedIndex;

            Boolean bResult = m_objVaxCOM.ConfAllowLine(nLineNo, CheckAllowListenConf.Checked, CheckAllowSpeakConf.Checked);

            if (!bResult)
                return;

            m_arrayAllowSpeakConf[nLineNo] = CheckAllowSpeakConf.Checked;
        }         

    }

    class cListIncomingCall  
    {
        public String m_sCallId;
        public String m_sCallerId;
        
        public cListIncomingCall(String sCallId, String sCallerId)
        {
            m_sCallId = sCallId;
            m_sCallerId = sCallerId;           
        }        

        public override String ToString()
        {
            return "CLI: " + m_sCallerId;
        }
    }
}
